import { NDKEvent } from '@nostr-dev-kit/ndk'
import { log } from '../main.js'
import axios from 'axios'

export async function blockChainBlockNumberJob(
  event: NDKEvent
): Promise<string> {
  log('New blockChain-block-number job', event.rawEvent())

  const input = event.tagValue('i')

  const blockChainUrl = `https://blockchain.info/blocks/${input}?format=json`

  const output = await axios
    .get(blockChainUrl)
    .then((res) => {
      const closestObject = findClosestObject(res.data, input)
      return closestObject.block_index.toString()
    })
    .catch((err) => {
      console.log('err in blockChain request :>> ', err)
      return Promise.reject(err)
    })

  return output
}

// todo: define type for array
// Function to find the object with the closest timestamp
function findClosestObject(array, timestamp) {
  let closestObject = null
  let minDifference = Infinity

  array.forEach((obj) => {
    const difference = Math.abs(obj.time - timestamp)
    if (difference < minDifference) {
      minDifference = difference
      closestObject = obj
    }
  })

  return closestObject
}
