import { NDKEvent } from '@nostr-dev-kit/ndk'
import { log } from '../main.js'
import axios from 'axios'
import { RelayInfo } from '../types/index.js'

export const relayInfoJob = async (event: NDKEvent): Promise<string> => {
  log('New relay-info job', event.rawEvent())

  const input: string = event.tagValue('i')

  let relays: string[] = []

  try {
    const relaysArr = JSON.parse(input)

    // input is a string containing an array of strings representing multiple relay URIs
    relays = relaysArr
  } catch (err) {
    // input is a string containing a string representing single relay URI

    relays.push(input)
  }

  const prefixes = { wss: 'wss://', https: 'https://', http: 'http://' }
  const headers = {
    headers: { Accept: 'application/nostr+json' }
  }

  const requests = relays.map((relay) => {
    if (relay.startsWith(prefixes.wss)) {
      relay = relay.replace(prefixes.wss, '')
    }

    return axios
      .get<RelayInfo>((relay = `${prefixes.https}${relay}`), headers)
      .catch(() => {
        return axios
          .get<RelayInfo>(`${prefixes.http}${relay}`, headers)
          .catch(() => undefined)
      })
  })

  let responses = await Promise.all(requests)
  responses = responses.filter((response) => response !== undefined)

  const relaysInfo: { [key: string]: RelayInfo } = {}

  responses.forEach((response) => {
    const relayURI = response.config.url
      .replace(prefixes.https, prefixes.wss)
      .replace(prefixes.http, prefixes.wss)

    relaysInfo[relayURI] = response.data
  })

  return Promise.resolve(JSON.stringify(relaysInfo))
}
